@@ -113,6 +113,8 @@ class Builder:
     def meta(self) -> Metadata:
         if not self._meta:
             self._meta = Metadata(self.location / "pyproject.toml")
+            # Open the validation for next release
+            self._meta.validate(False)
         return self._meta
 
     def __enter__(self) -> "Builder":
