@@ -2,7 +2,18 @@ import glob
 import os
 import re
 from pathlib import Path
-from typing import Any, Callable, Dict, Generic, List, Optional, Type, TypeVar, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generic,
+    List,
+    Mapping,
+    Optional,
+    Type,
+    TypeVar,
+    Union,
+)
 
 from pdm.pep517._vendor import toml
 from pdm.pep517._vendor.packaging.requirements import Requirement
@@ -19,6 +30,7 @@ from pdm.pep517.utils import (
     merge_marker,
     safe_name,
 )
+from pdm.pep517.validator import validate_pep621
 
 T = TypeVar("T")
 
@@ -65,26 +77,36 @@ class Metadata:
     DEFAULT_ENCODING = "utf-8"
     SUPPORTED_CONTENT_TYPES = ("text/markdown", "text/x-rst", "text/plain")
 
-    def __init__(self, filepath: Union[str, Path]) -> None:
+    def __init__(
+        self, filepath: Union[str, Path], data: Optional[Mapping] = None
+    ) -> None:
         self.filepath = Path(filepath).absolute()
-        self._metadata = self._read_pyproject(self.filepath)
+        self._tool_settings = {}
+        self._metadata = data
+        if self._metadata is None:
+            self._read_pyproject()
 
-    @staticmethod
-    def _read_pyproject(filepath: Path) -> Dict[str, Any]:
+    def _read_pyproject(self) -> Dict[str, Any]:
         try:
-            data = toml.loads(filepath.read_text(encoding="utf-8"))
+            data = toml.loads(self.filepath.read_text(encoding="utf-8"))
         except FileNotFoundError:
             raise ProjectError("pyproject.toml does not exist.")
         except toml.TomlDecodeError:
             raise ProjectError("The project's pyproject.toml is not valid.")
         else:
+            if "tool" in data and "pdm" in data["tool"]:
+                self._tool_settings = data["tool"]["pdm"]
             if "project" in data:
-                return data["project"]
-            elif "tool" in data and "pdm" in data["tool"]:
-                return convert_legacy(data["tool"]["pdm"])
+                self._metadata = data["project"]
+            elif self._tool_settings:
+                # TODO: deprecate legacy format
+                self._metadata = convert_legacy(self._tool_settings)
             else:
                 raise ProjectError("No [project] config in pyproject.toml")
 
+    def validate(self, raising: bool = False) -> bool:
+        return validate_pep621(self._metadata, raising)
+
     name: MetaField[str] = MetaField("name")
 
     def _get_version(self, value):
@@ -222,9 +244,42 @@ class Metadata:
 
     keywords: MetaField[str] = MetaField("keywords")
     project_urls: MetaField[Dict[str, str]] = MetaField("urls")
-    includes: MetaField[List[str]] = MetaField("includes")
-    excludes: MetaField[List[str]] = MetaField("excludes")
-    build: MetaField[str] = MetaField("build")
+
+    # Deprecate legacy metadata location
+    @property
+    def includes(self):
+        if "includes" in self._metadata:
+            return self._metadata["includes"]
+        elif "includes" in self._tool_settings:
+            return self._tool_settings["includes"]
+        return None
+
+    @property
+    def excludes(self):
+        if "excludes" in self._metadata:
+            return self._metadata["excludes"]
+        elif "excludes" in self._tool_settings:
+            return self._tool_settings["excludes"]
+        return None
+
+    @property
+    def build(self):
+        if "build" in self._metadata:
+            return self._metadata["build"]
+        elif "build" in self._tool_settings:
+            return self._tool_settings["build"]
+        return None
+
+    @property
+    def package_dir(self) -> str:
+        """A directory that will be used to looking for packages."""
+        if "package-dir" in self._metadata:
+            return self._metadata["package-dir"]
+        elif "package-dir" in self._tool_settings:
+            return self._tool_settings["package-dir"]
+        elif self.filepath.parent.joinpath("src").is_dir():
+            return "src"
+        return ""
 
     def _convert_dependencies(self, deps):
         return list(filter(None, map(ensure_pep440_req, deps)))
@@ -284,15 +339,6 @@ class Metadata:
                 result[plugin] = [f"{k} = {v}" for k, v in value.items()]
         return result
 
-    @property
-    def package_dir(self) -> str:
-        """A directory that will be used to looking for packages."""
-        if "package-dir" in self._metadata:
-            return self._metadata["package-dir"]
-        elif self.filepath.parent.joinpath("src").is_dir():
-            return "src"
-        return ""
-
     def convert_package_paths(self) -> Dict[str, Union[List, Dict]]:
         """Return a {package_dir, packages, package_data, exclude_package_data} dict."""
         packages = []
